from typing import List, Optional, Union

try:
    from typing import Literal
except ImportError:
    from typing_extensions import Literal
from .image_api import (
    AnimateAnyoneRequestBody,
    ImageToImageRequestBody,
    ImageToVideoRequestBody,
    TextToImageRequestBody,
    ControlNetRequestBody,
    QRCodeRequest,
)
from .api_client import FireworksClient

from PIL import Image
import httpx
import io
import os
from dataclasses import dataclass
import base64
import asyncio


@dataclass
class Answer:
    image: Optional[Image.Image]
    finish_reason: str


@dataclass
class AnswerVideo:
    video: bytes
    finish_reason: str


class ImageInference(FireworksClient):
    """
    Main client class for the Fireworks Image and Video Generation APIs.
    """

    def __init__(
        self,
        account: str = "fireworks",
        model: str = "stable-diffusion-xl-1024-v1-0",
        request_timeout=600,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        **kwargs,
    ) -> None:
        super().__init__(request_timeout, api_key=api_key, base_url=base_url, **kwargs)
        self.account = account
        self.model = model

    def _img_to_bytes(
        self, image: Union[Image.Image, str, os.PathLike, bytes]
    ) -> bytes:
        # Normalize all forms of `image` into a `bytes` object
        # to send over the wire
        if isinstance(image, Image.Image):
            img_bio = io.BytesIO()
            image.save(img_bio, format="PNG")
            image = img_bio.getvalue()
        elif isinstance(image, (str, os.PathLike)):
            with open(image, "rb") as f:
                image = f.read()

        return image

    def text_to_image(
        self,
        prompt: str,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        height: Optional[int] = None,
        width: Optional[int] = None,
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        safety_check: bool = False,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
        model: Optional[str] = None,
    ):
        """
        Generate an image or images based on the given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - prompt (str): The main text prompt based on which the image will be generated.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image generation in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image generation. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the generation process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the generated image. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.
        - model (str, optional): The model name to call. If not present, defaults to `self.model` on the ImageInference object.

        Returns:
        Image.Image or List[Image.Image]: Generated image or a list of generated images.

        Raises:
        RuntimeError: If there is an error in the image generation process.
        """
        return asyncio.run(
            self.text_to_image_async(
                prompt,
                negative_prompt,
                cfg_scale,
                clip_guidance_preset,
                height,
                width,
                sampler,
                samples,
                steps,
                seed,
                style_preset,
                safety_check,
                lora_adapter_name,
                lora_weight_filename,
                hf_access_key,
                output_image_format,
                model,
            )
        )

    async def text_to_image_async(
        self,
        prompt: str,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        height: Optional[int] = None,
        width: Optional[int] = None,
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        safety_check: bool = False,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
        model: Optional[str] = None,
    ) -> Union[Image.Image, List[Image.Image]]:
        """
        Generate an image or images based on the given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - prompt (str): The main text prompt based on which the image will be generated.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image generation in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image generation. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the generation process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the generated image. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.
        - model (str, optional): The model name to call. If not present, defaults to `self.model` on the ImageInference object.

        Returns:
        Image.Image or List[Image.Image]: Generated image or a list of generated images.

        Raises:
        RuntimeError: If there is an error in the image generation process.
        """
        request_body = TextToImageRequestBody(
            cfg_scale=cfg_scale,
            clip_guidance_preset=clip_guidance_preset,
            height=height,
            width=width,
            sampler=sampler,
            samples=samples,
            steps=steps,
            seed=seed,
            style_preset=style_preset,
            prompt=prompt,
            negative_prompt=negative_prompt,
            safety_check=safety_check,
            lora_adapter_name=lora_adapter_name,
            lora_weight_filename=lora_weight_filename,
        )
        payload_dict = request_body.dict()
        headers = {
            "Authorization": f"Bearer {self.api_key}",
            "Content-Type": "application/json",
        }
        if samples == 1:
            if output_image_format in {"JPG", "JPEG"}:
                headers["Accept"] = "image/jpeg"
            elif output_image_format == "PNG":
                headers["Accept"] = "image/png"
            else:
                raise ValueError(
                    f"Unsupported output_image_format: {output_image_format}"
                )
        else:
            if output_image_format == "PNG":
                headers["Accept"] = "application/json"
            else:
                raise ValueError(
                    f"Only PNG output image format is supported when samples != 1"
                )

        if hf_access_key is not None:
            headers["Huggingface-Access-Key"] = hf_access_key

        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/image_generation/accounts/{self.account}/models/{model or self.model}"
            response = await client.post(endpoint_base_uri, json=payload_dict)
            self._error_handling(response)
        if samples == 1:
            finish_reason = response.headers.get("finish-reason", "SUCCESS")
            return Answer(
                image=Image.open(io.BytesIO(response.content)),
                finish_reason=finish_reason,
            )
        else:
            return [
                Answer(
                    Image.open(io.BytesIO(base64.b64decode(artifact["base64"]))),
                    finish_reason=artifact["finishReason"],
                )
                for artifact in response.json()
            ]

    def image_to_image(
        self,
        init_image: Union[Image.Image, str, os.PathLike, bytes],
        prompt: str,
        height: Optional[int] = None,
        width: Optional[int] = None,
        crop_padding: bool = True,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        init_image_mode: str = "IMAGE_STRENGTH",
        image_strength: Optional[float] = None,
        step_schedule_start: Optional[float] = 0.65,
        step_schedule_end: Optional[float] = None,
        safety_check: bool = False,
        client_account_id_header: Optional[str] = None,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
        model: Optional[str] = None,
    ) -> Union[Image.Image, List[Image.Image]]:
        """
        Modify an existing image based on a given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - init_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - prompt (str): The main text prompt based on which the image will be modified.
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - crop_padding (bool, optional): Whether to crop the padding from the generated image. Defaults to True.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image modification in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image modification. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the modification process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the modified image. Optional.
        - init_image_mode (str, optional): Initialization mode for the image modification. Defaults to "IMAGE_STRENGTH".
        - image_strength (float, optional): Strength of the initial image. Required when init_image_mode is "IMAGE_STRENGTH".
        - step_schedule_start (float, optional): Start of the step schedule. Required when init_image_mode is "STEP_SCHEDULE". Defaults to 0.65.
        - step_schedule_end (float, optional): End of the step schedule. Required when init_image_mode is "STEP_SCHEDULE".
        - model (str, optional): which model to use. Defaults to "accounts/fireworks/models/stable-diffusion-xl-1024-v1-0".
        - client_account_id_header (str, optional): Client account ID header. Used for private deployments. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.
        - model (str, optional): The model name to call. If not present, defaults to `self.model` on the ImageInference object.

        Returns:
        Image.Image or List[Image.Image]: Modified image or a list of modified images.

        Raises:
        ValueError: If required parameters are missing based on the given init_image_mode.
        RuntimeError: If there is an error in the image modification process.
        """
        return asyncio.run(
            self.image_to_image_async(
                init_image,
                prompt,
                height,
                width,
                crop_padding,
                negative_prompt,
                cfg_scale,
                clip_guidance_preset,
                sampler,
                samples,
                steps,
                seed,
                style_preset,
                init_image_mode,
                image_strength,
                step_schedule_start,
                step_schedule_end,
                safety_check,
                client_account_id_header,
                lora_adapter_name,
                lora_weight_filename,
                hf_access_key,
                output_image_format,
                model=model,
            )
        )

    async def image_to_image_async(
        self,
        init_image: Union[Image.Image, str, os.PathLike, bytes],
        prompt: str,
        height: Optional[int] = None,
        width: Optional[int] = None,
        crop_padding: bool = True,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        init_image_mode: str = "IMAGE_STRENGTH",
        image_strength: Optional[float] = None,
        step_schedule_start: Optional[float] = 0.65,
        step_schedule_end: Optional[float] = None,
        safety_check: bool = False,
        client_account_id_header: Optional[str] = None,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
        model: Optional[str] = None,
    ) -> Union[Image.Image, List[Image.Image]]:
        """
        Modify an existing image based on a given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - init_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - prompt (str): The main text prompt based on which the image will be modified.
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - crop_padding (bool, optional): Whether to crop the padding from the generated image. Defaults to True.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image modification in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image modification. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the modification process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the modified image. Optional.
        - init_image_mode (str, optional): Initialization mode for the image modification. Defaults to "IMAGE_STRENGTH".
        - image_strength (float, optional): Strength of the initial image. Required when init_image_mode is "IMAGE_STRENGTH".
        - step_schedule_start (float, optional): Start of the step schedule. Required when init_image_mode is "STEP_SCHEDULE". Defaults to 0.65.
        - step_schedule_end (float, optional): End of the step schedule. Required when init_image_mode is "STEP_SCHEDULE".
        - model (str, optional): which model to use. Defaults to "accounts/fireworks/models/stable-diffusion-xl-1024-v1-0".
        - client_account_id_header (str, optional): Client account ID header. Used for private deployments. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.
        - model (str, optional): The model name to call. If not present, defaults to `self.model` on the ImageInference object.

        Returns:
        Image.Image or List[Image.Image]: Modified image or a list of modified images.

        Raises:
        ValueError: If required parameters are missing based on the given init_image_mode.
        RuntimeError: If there is an error in the image modification process.
        """
        # Argument Validation
        if init_image_mode == "IMAGE_STRENGTH" and image_strength is None:
            raise ValueError(
                "image_strength is required when init_image_mode is IMAGE_STRENGTH"
            )
        if init_image_mode == "STEP_SCHEDULE" and (
            step_schedule_start is None or step_schedule_end is None
        ):
            raise ValueError(
                "Both step_schedule_start and step_schedule_end are required when init_image_mode is STEP_SCHEDULE"
            )

        # Construct and validate request fields.
        # NB: prompt and init_image are not used here. Instead, we construct
        # them specially to be sent as multipart/form-data
        request_body = ImageToImageRequestBody(
            prompt=prompt,
            height=height,
            width=width,
            crop_padding=crop_padding,
            negative_prompt=negative_prompt,
            cfg_scale=cfg_scale,
            clip_guidance_preset=clip_guidance_preset,
            sampler=sampler,
            samples=samples,
            steps=steps,
            seed=seed,
            style_preset=style_preset,
            init_image_mode=init_image_mode,
            image_strength=image_strength,
            step_schedule_start=step_schedule_start,
            step_schedule_end=step_schedule_end,
            safety_check=safety_check,
            lora_adapter_name=lora_adapter_name,
            lora_weight_filename=lora_weight_filename,
        )
        payload_dict = request_body.dict()

        headers = {"Authorization": f"Bearer {self.api_key}"}
        if samples == 1:
            if output_image_format in {"JPG", "JPEG"}:
                headers["Accept"] = "image/jpeg"
            elif output_image_format == "PNG":
                headers["Accept"] = "image/png"
            else:
                raise ValueError(
                    f"Unsupported output_image_format: {output_image_format}"
                )
        else:
            if output_image_format == "PNG":
                headers["Accept"] = "application/json"
            else:
                raise ValueError(
                    f"Only PNG output image format is supported when samples != 1"
                )

        if client_account_id_header:
            headers["X-Fireworks-Routing-Account-ID"] = client_account_id_header

        if hf_access_key is not None:
            headers["Huggingface-Access-Key"] = hf_access_key

        init_image: bytes = self._img_to_bytes(init_image)

        files = {
            "init_image": init_image,
        }
        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/image_generation/accounts/{self.account}/models/{model or self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/image_to_image",
                data=payload_dict,
                files=files,
            )
            self._error_handling(response)
        if samples == 1:
            finish_reason = response.headers.get("finish-reason", "SUCCESS")
            return Answer(
                image=Image.open(io.BytesIO(response.content)),
                finish_reason=finish_reason,
            )
        else:
            return [
                Answer(
                    Image.open(io.BytesIO(base64.b64decode(artifact["base64"]))),
                    finish_reason=artifact["finishReason"],
                )
                for artifact in response.json()
            ]

    def control_net(
        self,
        control_image: Union[Image.Image, str, os.PathLike, bytes],
        control_net_name: str,
        prompt: str,
        height: Optional[int] = None,
        width: Optional[int] = None,
        crop_padding: bool = True,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        step_schedule_start: Optional[float] = 0.0,
        step_schedule_end: Optional[float] = 1.0,
        conditioning_scale: Optional[float] = 0.5,
        safety_check: bool = False,
        client_account_id_header: Optional[str] = None,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
    ) -> Union[Image.Image, List[Image.Image]]:
        """
        Modify an existing image based on a given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - control_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - control_net_name (str): Name of HuggingFace repository with control net name or a simple reference like "canny".
        - prompt (str): The main text prompt based on which the image will be modified.
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - crop_padding (bool, optional): Whether to crop the padding from the generated image. Defaults to True.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image modification in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image modification. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the modification process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the modified image. Optional.
        - step_schedule_start (float, optional): The percentage of total steps at which the ControlNet starts applying.
        - step_schedule_end (float, optional): The percentage of total steps at which the ControlNet stops applying.
        - conditioning_scale (float, optional, defaults to 0.5): The outputs of the ControlNet are multiplied by this value before they are added to the residual.
        - model (str, optional): which model to use. Defaults to "accounts/fireworks/models/stable-diffusion-xl-1024-v1-0".
        - client_account_id_header (str, optional): Client account ID header. Used for private deployments. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.

        Returns:
        Image.Image or List[Image.Image]: Modified image or a list of modified images.

        Raises:
        RuntimeError: If there is an error in the image generation process.
        """
        return asyncio.run(
            self.control_net_async(
                control_image,
                control_net_name,
                prompt,
                height,
                width,
                crop_padding,
                negative_prompt,
                cfg_scale,
                clip_guidance_preset,
                sampler,
                samples,
                steps,
                seed,
                style_preset,
                step_schedule_start,
                step_schedule_end,
                conditioning_scale,
                safety_check,
                client_account_id_header,
                lora_adapter_name,
                lora_weight_filename,
                hf_access_key,
                output_image_format,
            )
        )

    async def control_net_async(
        self,
        control_image: Union[Image.Image, str, os.PathLike, bytes],
        control_net_name: str,
        prompt: str,
        height: Optional[int] = None,
        width: Optional[int] = None,
        crop_padding: bool = True,
        negative_prompt: Optional[str] = None,
        cfg_scale: Optional[int] = None,
        clip_guidance_preset: str = "NONE",
        sampler: Optional[str] = None,
        samples: int = 1,
        steps: Optional[int] = None,
        seed: int = 0,
        style_preset: Optional[str] = None,
        step_schedule_start: Optional[float] = 0.0,
        step_schedule_end: Optional[float] = 1.0,
        conditioning_scale: Optional[float] = 0.5,
        safety_check: bool = False,
        client_account_id_header: Optional[str] = None,
        lora_adapter_name: Optional[str] = None,
        lora_weight_filename: Optional[str] = None,
        hf_access_key: Optional[str] = None,
        output_image_format: str = "PNG",
    ) -> Union[Image.Image, List[Image.Image]]:
        """
        Modify an existing image based on a given text prompt and optional negative prompt.
        See the OpenAPI spec (https://readme.fireworks.ai/reference/post_image-generation-stable-diffusion)
        for the most up-to-date description of the supported parameters

        Parameters:
        - control_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - control_net_name (str): Name of HuggingFace repository with control net name or a simple reference like "canny".
        - prompt (str): The main text prompt based on which the image will be modified.
        - height (int, optional): Desired height of the generated image. If not specified, a model-specific default will be used.
        - width (int, optional): Desired width of the generated image. If not used, a model-specific default will be used.
        - crop_padding (bool, optional): Whether to crop the padding from the generated image. Defaults to True.
        - negative_prompt (str, optional): A secondary text prompt which can be used to guide the image modification in a negative way.
        - cfg_scale (int, optional): Context-free guidance scale for the image modification. If not specified, a model-specific default will be used
        - clip_guidance_preset (str, optional): CLIP guidance preset. Defaults to "NONE".
        - sampler (str, optional): Sampler type. Optional.
        - samples (int, optional): Number of images to be generated. Defaults to 1.
        - steps (int, optional): Number of steps for the modification process. If not specified, a model-specific default will be used.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - style_preset (str, optional): Style preset for the modified image. Optional.
        - step_schedule_start (float, optional): The percentage of total steps at which the ControlNet starts applying.
        - step_schedule_end (float, optional): The percentage of total steps at which the ControlNet stops applying.
        - conditioning_scale (float, optional, defaults to 0.5): The outputs of the ControlNet are multiplied by this value before they are added to the residual.
        - model (str, optional): which model to use. Defaults to "accounts/fireworks/models/stable-diffusion-xl-1024-v1-0".
        - client_account_id_header (str, optional): Client account ID header. Used for private deployments. Optional.
        - lora_adapter_name (str, optional): LoRA adapter name. This can refer to a fully-qualified HuggingFace repo name, e.g. 'foo/bar'. Optional
        - lora_weight_filename (str, optional): The filename within the LoRA repo to load LoRA weights from. Should be a single filename like 'pytorch_lora_weights.safetensors'. Optional. Must be specified if lora_adapter_name is specified.
        - hf_access_key (str, optional): The access key to access HuggingFace to download a LoRA adapter. Specify if your adapter is in a private repo that requires permissions. Optional.
        - output_image_format (str): The format of the generated image. Defaults to "PNG". Options are "PNG" and "JPG". Use PNG for best quality and JPG for best speed. JPG currently only supported for samples=1.

        Returns:
        Image.Image or List[Image.Image]: Modified image or a list of modified images.

        Raises:
        RuntimeError: If there is an error in the image generation process.
        """
        # Argument Validation
        # Construct and validate request fields.
        # NB: prompt and control_image are not used here. Instead, we construct
        # them specially to be sent as multipart/form-data
        request_body = ControlNetRequestBody(
            prompt=prompt,
            height=height,
            width=width,
            crop_padding=crop_padding,
            negative_prompt=negative_prompt,
            cfg_scale=cfg_scale,
            clip_guidance_preset=clip_guidance_preset,
            sampler=sampler,
            samples=samples,
            steps=steps,
            seed=seed,
            style_preset=style_preset,
            control_net_name=control_net_name,
            step_schedule_start=step_schedule_start,
            step_schedule_end=step_schedule_end,
            conditioning_scale=conditioning_scale,
            safety_check=safety_check,
            lora_adapter_name=lora_adapter_name,
            lora_weight_filename=lora_weight_filename,
        )
        payload_dict = request_body.dict()

        headers = {"Authorization": f"Bearer {self.api_key}"}
        if samples == 1:
            headers["Accept"] = "image/png"
        else:
            headers["Accept"] = "application/json"
        if samples == 1:
            if output_image_format in {"JPG", "JPEG"}:
                headers["Accept"] = "image/jpeg"
            elif output_image_format == "PNG":
                headers["Accept"] = "image/png"
            else:
                raise ValueError(
                    f"Unsupported output_image_format: {output_image_format}"
                )
        else:
            if output_image_format == "PNG":
                headers["Accept"] = "application/json"
            else:
                raise ValueError(
                    f"Only PNG output image format is supported when samples != 1"
                )

        if client_account_id_header:
            headers["X-Fireworks-Routing-Account-ID"] = client_account_id_header

        if hf_access_key is not None:
            headers["Huggingface-Access-Key"] = hf_access_key

        control_image: bytes = self._img_to_bytes(control_image)

        files = {
            "control_image": control_image,
        }
        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/image_generation/accounts/{self.account}/models/{self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/control_net",
                data=payload_dict,
                files=files,
            )
            self._error_handling(response)
        if samples == 1:
            finish_reason = response.headers.get("finish-reason", "SUCCESS")
            return Answer(
                image=Image.open(io.BytesIO(response.content)),
                finish_reason=finish_reason,
            )
        else:
            return [
                Answer(
                    Image.open(io.BytesIO(base64.b64decode(artifact["base64"]))),
                    finish_reason=artifact["finishReason"],
                )
                for artifact in response.json()
            ]

    def canny_edge_detection(
        self,
        image: Union[Image.Image, str, os.PathLike, bytes],
        min_val: float = 100,
        max_val: float = 200,
    ) -> Image.Image:
        """
        Detect edges in an image using Canny edge detection.

        Parameters:
        - image (Union[Image.Image, str, os.PathLike, bytes]): Image to detect edges in. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - min_val (float, optional): Minimum threshold value. Defaults to 100.
        - max_val (float, optional): Maximum threshold value. Defaults to 200.

        Returns:
        Image.Image: Image with detected edges.

        Raises:
        RuntimeError: If there is an error in the edge detection process.
        """
        return asyncio.run(
            self.canny_edge_detection_async(
                image,
                min_val,
                max_val,
            )
        )

    async def canny_edge_detection_async(
        self,
        image: Union[Image.Image, str, os.PathLike, bytes],
        min_val: float = 100,
        max_val: float = 200,
    ) -> Image.Image:
        """
        Detect edges in an image using Canny edge detection.

        Parameters:
        - image (Union[Image.Image, str, os.PathLike, bytes]): Image to detect edges in. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - min_val (float, optional): Minimum threshold value. Defaults to 100.
        - max_val (float, optional): Maximum threshold value. Defaults to 200.

        Returns:
        Image.Image: Image with detected edges.

        Raises:
        RuntimeError: If there is an error in the edge detection process.
        """
        image: bytes = self._img_to_bytes(image)

        headers = {"Authorization": f"Bearer {self.api_key}"}

        files = {
            "image": image,
        }
        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/image_generation/accounts/{self.account}/models/{self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/canny_edge_detection",
                files=files,
                data={"min_val": min_val, "max_val": max_val},
            )
            self._error_handling(response)

        return Image.open(io.BytesIO(response.content))

    def qr_code(
        self,
        prompt: str,
        height: int = 1024,
        width: int = 1024,
        output_image_format: str = "PNG",
    ) -> Image.Image:
        """
        Generate a QR code based on the given text prompt.

        Parameters:
        - prompt (str): The text prompt based on which the QR code will be generated.
        - height (int, optional): Desired height of the generated QR code. Defaults to 1024.
        - width (int, optional): Desired width of the generated QR code. Defaults to 1024.

        Returns:
        Image.Image: Generated QR code.

        Raises:
        RuntimeError: If there is an error in the QR code generation process.
        """
        return asyncio.run(
            self.qr_code_async(
                prompt,
                height,
                width,
                output_image_format,
            )
        )

    async def qr_code_async(
        self,
        prompt: str,
        height: int = 1024,
        width: int = 1024,
        output_image_format: str = "PNG",
    ) -> Image.Image:
        """
        Generate a QR code based on the given text prompt.

        Parameters:
        - prompt (str): The text prompt based on which the QR code will be generated.
        - height (int, optional): Desired height of the generated QR code. Defaults to 1024.
        - width (int, optional): Desired width of the generated QR code. Defaults to 1024.

        Returns:
        Image.Image: Generated QR code.

        Raises:
        RuntimeError: If there is an error in the QR code generation process.
        """
        headers = {"Authorization": f"Bearer {self.api_key}"}

        if output_image_format in {"JPG", "JPEG"}:
            headers["Accept"] = "image/jpeg"
        elif output_image_format == "PNG":
            headers["Accept"] = "image/png"
        else:
            raise ValueError(f"Unsupported output_image_format: {output_image_format}")

        body = QRCodeRequest(
            prompt=prompt,
            height=height,
            width=width,
        )

        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/image_generation/accounts/{self.account}/models/{self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/qr_code",
                json=body.dict(),
            )
            self._error_handling(response)

        return Image.open(io.BytesIO(response.content))

    async def image_to_video_async(
        self,
        input_image: Union[Image.Image, str, os.PathLike, bytes],
        height: int = 576,
        width: int = 1024,
        sampler: Optional[str] = None,
        frames: Optional[int] = None,
        steps: int = 25,
        min_guidance_scale: float = 1.0,
        max_guidance_scale: float = 3.0,
        fps: int = 7,
        motion_bucket_id: int = 127,
        noise_aug_strength: float = 0.02,
        decode_chunk_size: Optional[int] = None,
        seed: int = 0,
        safety_check: bool = False,
        frame_interpolation_factor: Optional[int] = None,
        output_video_bitrate: Optional[int] = 8_388_608,  # Default 8Mbps
        _infer_if_input_is_flagged_DEBUG_ONLY: bool = False,
    ) -> AnswerVideo:
        """
        Generate a video based on a given initial image.

        Parameters:
        - init_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - height (int, optional): Desired height of the generated video. Defaults to 576.
        - width (int, optional): Desired width of the generated video. Defaults to 1024.
        - sampler (str, optional): Sampler type. Optional.
        - frames (int, optional): Number of frames to be generated. Defaults to None.
        - steps (int, optional): Number of steps for the modification process. Defaults to 25.
        - min_guidance_scale (float, optional): Minimum guidance scale. Defaults to 1.0.
        - max_guidance_scale (float, optional): Maximum guidance scale. Defaults to 3.0.
        - fps (int, optional): Frames per second. Defaults to 7.
        - motion_bucket_id (int, optional): Motion bucket ID. Defaults to 127.
        - noise_aug_strength (float, optional): Noise augmentation strength. Defaults to 0.02.
        - decode_chunk_size (int, optional): Decode chunk size. Defaults to None.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - safety_check (bool, optional): Whether to perform a safety check on the input image and generated frames. Defaults to False.
        - frame_interpolation_factor(int, optional): Factor for frame interpolation. For example, a factor of 2 will produce 2x more frames, 3 3x more frames, and so on.
        - output_video_bitrate (int, optional): Bitrate of the generated video. Defaults to 8Mbps.

        Returns:
        bytes: Generated video.

        Raises:
        RuntimeError: If there is an error in the video generation process.
        """
        # Construct and validate request fields.
        # NB: init_image is not used here. Instead, we construct
        # it specially to be sent as multipart/form-data
        request_body = ImageToVideoRequestBody(
            height=height,
            width=width,
            sampler=sampler,
            frames=frames,
            steps=steps,
            min_guidance_scale=min_guidance_scale,
            max_guidance_scale=max_guidance_scale,
            fps=fps,
            motion_bucket_id=motion_bucket_id,
            noise_aug_strength=noise_aug_strength,
            decode_chunk_size=decode_chunk_size,
            seed=seed,
            safety_check=safety_check,
            frame_interpolation_factor=frame_interpolation_factor,
            output_video_bitrate=output_video_bitrate,
            infer_if_input_is_flagged_DEBUG_ONLY=_infer_if_input_is_flagged_DEBUG_ONLY,
        )
        payload_dict = request_body.dict()

        headers = {"Authorization": f"Bearer {self.api_key}"}
        headers["Accept"] = "video/mp4"

        input_image: bytes = self._img_to_bytes(input_image)

        files = {
            "input_image": input_image,
        }
        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/video_generation/accounts/{self.account}/models/{self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/image_to_video",
                data=payload_dict,
                files=files,
            )
            self._error_handling(response)

        finish_reason = response.headers.get("finish-reason", "SUCCESS")
        return AnswerVideo(
            video=response.content,
            finish_reason=finish_reason,
        )

    def image_to_video(
        self,
        input_image: Union[Image.Image, str, os.PathLike, bytes],
        height: int = 576,
        width: int = 1024,
        sampler: Optional[str] = None,
        frames: Optional[int] = None,
        steps: int = 25,
        min_guidance_scale: float = 1.0,
        max_guidance_scale: float = 3.0,
        fps: int = 7,
        motion_bucket_id: int = 127,
        noise_aug_strength: float = 0.02,
        decode_chunk_size: Optional[int] = None,
        seed: int = 0,
        safety_check: bool = False,
        frame_interpolation_factor: Optional[int] = None,
        output_video_bitrate: Optional[int] = 8_388_608,  # Default 8Mbps
        _infer_if_input_is_flagged_DEBUG_ONLY: bool = False,
    ) -> AnswerVideo:
        """
        Generate a video based on a given initial image.

        Parameters:
        - input_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - height (int, optional): Desired height of the generated video. Defaults to 576.
        - width (int, optional): Desired width of the generated video. Defaults to 1024.
        - sampler (str, optional): Sampler type. Optional.
        - frames (int, optional): Number of frames to be generated. Defaults to None.
        - steps (int, optional): Number of steps for the modification process. Defaults to 25.
        - min_guidance_scale (float, optional): Minimum guidance scale. Defaults to 1.0.
        - max_guidance_scale (float, optional): Maximum guidance scale. Defaults to 3.0.
        - fps (int, optional): Frames per second. Defaults to 7.
        - motion_bucket_id (int, optional): Motion bucket ID. Defaults to 127.
        - noise_aug_strength (float, optional): Noise augmentation strength. Defaults to 0.02.
        - decode_chunk_size (int, optional): Decode chunk size. Defaults to None.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - safety_check (bool, optional): Whether to perform a safety check on the input image and generated frames. Defaults to False.
        - frame_interpolation_factor(int, optional): Factor for frame interpolation. For example, a factor of 2 will produce 2x more frames, 3 3x more frames, and so on.
        - output_video_bitrate (int, optional): Bitrate of the generated video. Defaults to 8Mbps.

        Returns:
        bytes: Generated video.

        Raises:
        RuntimeError: If there is an error in the video generation process.
        """
        return asyncio.run(
            self.image_to_video_async(
                input_image,
                height,
                width,
                sampler,
                frames,
                steps,
                min_guidance_scale,
                max_guidance_scale,
                fps,
                motion_bucket_id,
                noise_aug_strength,
                decode_chunk_size,
                seed,
                safety_check,
                frame_interpolation_factor,
                output_video_bitrate=output_video_bitrate,
                _infer_if_input_is_flagged_DEBUG_ONLY=_infer_if_input_is_flagged_DEBUG_ONLY,
            )
        )

    def _audio_to_bytes(self, input_audio: Union[bytes, os.PathLike]) -> bytes:
        # NB: assuming "bytes" is raw input data. If the user wants to
        # specify a file path, they should pass in a string
        if isinstance(input_audio, (str, os.PathLike)):
            with open(input_audio, "rb") as f:
                input_audio = f.read()

        return input_audio

    async def animate_async(
        self,
        ref_image: Union[Image.Image, str, os.PathLike, bytes],
        input_audio: Union[bytes, os.PathLike],
        width: int = 512,
        height: int = 512,
        steps: int = 10,
        audio_guidance_scale: float = 3.0,
        context_guidance_scale: float = 0.0,
        guidance_rescale: float = 0.7,
        seed: int = 0,
        output_video_bitrate: int = 8_388_608,
        _fake_request: bool = False,
    ):
        """
        Generate a video based on a given initial image and speech audio track.

        Parameters:
        - ref_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - input_audio (Union[bytes, os.PathLike]): Input audio file to use for speech input. Can be raw audio file bytes or a file path.
        - width (int, optional): Desired width of the generated video. Defaults to 1024.
        - height (int, optional): Desired height of the generated video. Defaults to 576.
        - steps (int, optional): Number of steps for the modification process. Defaults to 25.
        - audio_guidance_scale (float, optional): Guidance scale to use for audio context. Defaults to 3.0.
        - context_guidance_scale (float, optional): Guidance scale to use for context. Defaults to 0.0.
        - guidance_rescale (float, optional): Rescaling factor for guidance. Defaults to 0.7.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - output_video_bitrate (int, optional): Bitrate of the generated video. Defaults to 8Mbps.

        Returns:
        bytes: Generated video.

        Raises:
        RuntimeError: If there is an error in the video generation process.
        """
        # Construct and validate request fields.
        # NB: ref_image and input_audio are not used here. Instead, we send those
        # as separate form fields
        request_body = AnimateAnyoneRequestBody(
            width=width,
            height=height,
            steps=steps,
            audio_guidance_scale=audio_guidance_scale,
            context_guidance_scale=context_guidance_scale,
            guidance_rescale=guidance_rescale,
            seed=seed,
            output_video_bitrate=output_video_bitrate,
        )
        payload_dict = request_body.dict()

        headers = {"Authorization": f"Bearer {self.api_key}"}
        headers["Accept"] = "video/mp4"
        if _fake_request:
            headers["Fireworks-Test-Request"] = "true"

        ref_image: bytes = self._img_to_bytes(ref_image)
        input_audio: bytes = self._audio_to_bytes(input_audio)

        files = {
            "ref_image": ref_image,
            "input_audio": input_audio,
        }
        async with httpx.AsyncClient(
            headers=headers,
            timeout=self.request_timeout,
            **self.client_kwargs,
        ) as client:
            endpoint_base_uri = f"{self.base_url}/video_generation/accounts/{self.account}/models/{self.model}"
            response = await client.post(
                f"{endpoint_base_uri}/animate",
                data=payload_dict,
                files=files,
            )
            self._error_handling(response)

        finish_reason = response.headers.get("finish-reason", "SUCCESS")
        return AnswerVideo(
            video=response.content,
            finish_reason=finish_reason,
        )

    def animate(
        self,
        ref_image: Union[Image.Image, str, os.PathLike, bytes],
        input_audio: Union[bytes, os.PathLike],
        width: int = 512,
        height: int = 512,
        steps: int = 10,
        audio_guidance_scale: float = 3.0,
        context_guidance_scale: float = 0.0,
        guidance_rescale: float = 0.7,
        seed: int = 0,
        output_video_bitrate: int = 8_388_608,
        _fake_request: bool = False,
    ):
        """
        Generate a video based on a given initial image and speech audio track.

        Parameters:
        - ref_image (Union[Image.Image, str, os.PathLike, bytes]): Initial image to be modified. It can be provided as a PIL Image object, path to an image, or raw bytes.
        - input_audio (Union[bytes, os.PathLike]): Input audio file to use for speech input. Can be raw audio file bytes or a file path.
        - width (int, optional): Desired width of the generated video. Defaults to 1024.
        - height (int, optional): Desired height of the generated video. Defaults to 576.
        - steps (int, optional): Number of steps for the modification process. Defaults to 25.
        - audio_guidance_scale (float, optional): Guidance scale to use for audio context. Defaults to 3.0.
        - context_guidance_scale (float, optional): Guidance scale to use for context. Defaults to 0.0.
        - guidance_rescale (float, optional): Rescaling factor for guidance. Defaults to 0.7.
        - seed (int, optional): Seed for random number generation. Defaults to 0.
        - output_video_bitrate (int, optional): Bitrate of the generated video. Defaults to 8Mbps.

        Returns:
        bytes: Generated video.

        Raises:
        RuntimeError: If there is an error in the video generation process.
        """
        return asyncio.run(
            self.animate_async(
                ref_image=ref_image,
                input_audio=input_audio,
                width=width,
                height=height,
                steps=steps,
                audio_guidance_scale=audio_guidance_scale,
                context_guidance_scale=context_guidance_scale,
                guidance_rescale=guidance_rescale,
                seed=seed,
                output_video_bitrate=output_video_bitrate,
                _fake_request=_fake_request,
            )
        )



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/eagle_training_job.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/eagle_training_job.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from ..buf.validate import validate_pb2 as buf_dot_validate_dot_validate__pb2
from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import options_pb2 as gateway_dot_options__pb2
from . import status_pb2 as gateway_dot_status__pb2
from . import training_pb2 as gateway_dot_training__pb2
from . import wandb_pb2 as gateway_dot_wandb__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from ..google.api import visibility_pb2 as google_dot_api_dot_visibility__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n gateway/eagle_training_job.proto\x12\x07gateway\x1a\x1b\x62uf/validate/validate.proto\x1a\x18gateway/deployment.proto\x1a\x15gateway/options.proto\x1a\x14gateway/status.proto\x1a\x16gateway/training.proto\x1a\x13gateway/wandb.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a\x1bgoogle/api/visibility.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb8\x07\n\x10\x45\x61gleTrainingJob\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x1a\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x01\x12\x36\n\x0b\x63reate_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x35\n\x0b\x65xpire_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12\x19\n\ncreated_by\x18\x05 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\'\n\x05state\x18\x06 \x01(\x0e\x32\x11.gateway.JobStateB\x05\xe2\x41\x02\x03\x05\x12&\n\x06status\x18\x07 \x01(\x0b\x32\x0f.gateway.StatusB\x05\xe2\x41\x02\x03\x05\x12\x1f\n\x11input_draft_model\x18\x08 \x01(\tB\x04\xe2\x41\x01\x02\x12\x1e\n\x10training_dataset\x18\t \x01(\tB\x04\xe2\x41\x01\x02\x12\x18\n\nbase_model\x18\n \x01(\tB\x04\xe2\x41\x01\x02\x12 \n\x12output_draft_model\x18\x0b \x01(\tB\x04\xe2\x41\x01\x02\x12)\n\x0c\x65pochs_count\x18\x0c \x01(\x02\x42\x13\xe2\x41\x01\x01\xbaH\x0c\n\n\x1d\x00\x00\xc8\x42-\x00\x00\x00\x00\x12*\n\rlearning_rate\x18\r \x01(\x02\x42\x13\xe2\x41\x01\x01\xbaH\x0c\n\n\x1d\xcd\xcc\xcc=-\x00\x00\x00\x00\x12\x30\n\x0cwandb_config\x18\x0e \x01(\x0b\x32\x14.gateway.WandbConfigB\x04\xe2\x41\x01\x01\x12\x39\n\x11\x65\x61rly_stop_config\x18\x0f \x01(\x0b\x32\x18.gateway.EarlyStopConfigB\x04\xe2\x41\x01\x01\x12N\n\x10\x61\x63\x63\x65lerator_type\x18\x14 \x01(\x0e\x32\x18.gateway.AcceleratorTypeB\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x35\n\x11\x61\x63\x63\x65lerator_count\x18\x15 \x01(\x05\x42\x1a\xe2\x41\x01\x01\xfa\xd2\xe4\x93\x02\x10\x12\x0eSUPERUSER_ONLY\x12\x35\n\x0bupdate_time\x18\x16 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03:\x98\x01\xea\x41\x85\x01\n!api.fireworks.ai/EagleTrainingJob\x12;accounts/{AccountId}/eagleTrainingJobs/{EagleTrainingJobId}*\x11\x65\x61gleTrainingJobs2\x10\x65\x61gleTrainingJob\x82\xf1\x04\x0b\n\x07\x41\x63\x63ount\x18\x01\"0\n\x1aGetEagleTrainingJobRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\"\x97\x01\n\x1d\x43reateEagleTrainingJobRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12;\n\x12\x65\x61gle_training_job\x18\x02 \x01(\x0b\x32\x19.gateway.EagleTrainingJobB\x04\xe2\x41\x01\x02\x12#\n\x15\x65\x61gle_training_job_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\"\x95\x01\n\x1cListEagleTrainingJobsRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\"\x84\x01\n\x1dListEagleTrainingJobsResponse\x12\x36\n\x13\x65\x61gle_training_jobs\x18\x01 \x03(\x0b\x32\x19.gateway.EagleTrainingJob\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"\x93\x01\n\x1dUpdateEagleTrainingJobRequest\x12;\n\x12\x65\x61gle_training_job\x18\x01 \x01(\x0b\x32\x19.gateway.EagleTrainingJobB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"3\n\x1d\x44\x65leteEagleTrainingJobRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.eagle_training_job_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_EAGLETRAININGJOB'].fields_by_name['name']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_EAGLETRAININGJOB'].fields_by_name['display_name']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['display_name']._serialized_options = b'\342A\001\001'
  _globals['_EAGLETRAININGJOB'].fields_by_name['create_time']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_EAGLETRAININGJOB'].fields_by_name['expire_time']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['expire_time']._serialized_options = b'\342A\001\003'
  _globals['_EAGLETRAININGJOB'].fields_by_name['created_by']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005'
  _globals['_EAGLETRAININGJOB'].fields_by_name['state']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['state']._serialized_options = b'\342A\002\003\005'
  _globals['_EAGLETRAININGJOB'].fields_by_name['status']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['status']._serialized_options = b'\342A\002\003\005'
  _globals['_EAGLETRAININGJOB'].fields_by_name['input_draft_model']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['input_draft_model']._serialized_options = b'\342A\001\002'
  _globals['_EAGLETRAININGJOB'].fields_by_name['training_dataset']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['training_dataset']._serialized_options = b'\342A\001\002'
  _globals['_EAGLETRAININGJOB'].fields_by_name['base_model']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['base_model']._serialized_options = b'\342A\001\002'
  _globals['_EAGLETRAININGJOB'].fields_by_name['output_draft_model']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['output_draft_model']._serialized_options = b'\342A\001\002'
  _globals['_EAGLETRAININGJOB'].fields_by_name['epochs_count']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['epochs_count']._serialized_options = b'\342A\001\001\272H\014\n\n\035\000\000\310B-\000\000\000\000'
  _globals['_EAGLETRAININGJOB'].fields_by_name['learning_rate']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['learning_rate']._serialized_options = b'\342A\001\001\272H\014\n\n\035\315\314\314=-\000\000\000\000'
  _globals['_EAGLETRAININGJOB'].fields_by_name['wandb_config']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['wandb_config']._serialized_options = b'\342A\001\001'
  _globals['_EAGLETRAININGJOB'].fields_by_name['early_stop_config']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['early_stop_config']._serialized_options = b'\342A\001\001'
  _globals['_EAGLETRAININGJOB'].fields_by_name['accelerator_type']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['accelerator_type']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EAGLETRAININGJOB'].fields_by_name['accelerator_count']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['accelerator_count']._serialized_options = b'\342A\001\001\372\322\344\223\002\020\022\016SUPERUSER_ONLY'
  _globals['_EAGLETRAININGJOB'].fields_by_name['update_time']._loaded_options = None
  _globals['_EAGLETRAININGJOB'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_EAGLETRAININGJOB']._loaded_options = None
  _globals['_EAGLETRAININGJOB']._serialized_options = b'\352A\205\001\n!api.fireworks.ai/EagleTrainingJob\022;accounts/{AccountId}/eagleTrainingJobs/{EagleTrainingJobId}*\021eagleTrainingJobs2\020eagleTrainingJob\202\361\004\013\n\007Account\030\001'
  _globals['_GETEAGLETRAININGJOBREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETEAGLETRAININGJOBREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job']._loaded_options = None
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job']._serialized_options = b'\342A\001\002'
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job_id']._loaded_options = None
  _globals['_CREATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job_id']._serialized_options = b'\342A\001\001'
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTEAGLETRAININGJOBSREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job']._loaded_options = None
  _globals['_UPDATEEAGLETRAININGJOBREQUEST'].fields_by_name['eagle_training_job']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEEAGLETRAININGJOBREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEEAGLETRAININGJOBREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEEAGLETRAININGJOBREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEEAGLETRAININGJOBREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_EAGLETRAININGJOB']._serialized_start=347
  _globals['_EAGLETRAININGJOB']._serialized_end=1299
  _globals['_GETEAGLETRAININGJOBREQUEST']._serialized_start=1301
  _globals['_GETEAGLETRAININGJOBREQUEST']._serialized_end=1349
  _globals['_CREATEEAGLETRAININGJOBREQUEST']._serialized_start=1352
  _globals['_CREATEEAGLETRAININGJOBREQUEST']._serialized_end=1503
  _globals['_LISTEAGLETRAININGJOBSREQUEST']._serialized_start=1506
  _globals['_LISTEAGLETRAININGJOBSREQUEST']._serialized_end=1655
  _globals['_LISTEAGLETRAININGJOBSRESPONSE']._serialized_start=1658
  _globals['_LISTEAGLETRAININGJOBSRESPONSE']._serialized_end=1790
  _globals['_UPDATEEAGLETRAININGJOBREQUEST']._serialized_start=1793
  _globals['_UPDATEEAGLETRAININGJOBREQUEST']._serialized_end=1940
  _globals['_DELETEEAGLETRAININGJOBREQUEST']._serialized_start=1942
  _globals['_DELETEEAGLETRAININGJOBREQUEST']._serialized_end=1993
# @@protoc_insertion_point(module_scope)



import io
from typing import Dict, Generator, List, Type, Union
import json
from pydantic import BaseModel
from .api_client import FireworksClient
from .api import (
    ChatCompletionResponse,
    ChatCompletionStreamResponse,
    CompletionResponse,
    CompletionStreamResponse,
)


def _non_streaming_call(
    sagemaker_runtime,
    endpoint: str,
    inference_id: str,
    body: dict,
):
    assert body["stream"] == False
    args = {
        "EndpointName": endpoint,
        "ContentType": "application/json",
        "Body": bytes(json.dumps(body), "utf-8"),
    }
    if inference_id is not None:
        args["InferenceId"] = inference_id
    response = sagemaker_runtime.invoke_endpoint(**args)
    return json.loads(response["Body"].read().decode("utf-8"))


class _StreamIterator:
    """
    A helper class for parsing the byte stream input.

    Adapted from https://aws.amazon.com/blogs/machine-learning/elevating-the-generative-ai-experience-introducing-streaming-support-in-amazon-sagemaker-hosting/

    The output of the model will be in the following format:
    ```
    b'data: {"outputs": [" a"]}\n'
    b'data: {"outputs": [" challenging"]}\n'
    b'data: {"outputs": [" problem"]}\n'
    b'data: DONE\n'
    ...
    ```

    While usually each PayloadPart event from the event stream will contain a byte array
    with a full json, this is not guaranteed and some of the json objects may be split across
    PayloadPart events. For example:
    ```
    {'PayloadPart': {'Bytes': b'data: {"outputs": '}}
    {'PayloadPart': {'Bytes': b'[" problem"]}\n'}}
    ```

    This class accounts for this by concatenating bytes written via the 'write' function
    and then exposing a method which will return lines (ending with a '\n' character) within
    the buffer via the 'scan_lines' function. It maintains the position of the last read
    position to ensure that previous bytes are not exposed again.
    """

    def __init__(self, stream):
        self.byte_iterator = iter(stream)
        self.buffer = io.BytesIO()
        self.read_pos = 0

    def __iter__(self):
        return self

    def __next__(self):
        while True:
            self.buffer.seek(self.read_pos)
            line = self.buffer.readline()
            if line and line[-1] == ord("\n"):
                self.read_pos += len(line)
                line = line.strip()
                if line == b"":
                    continue
                if not line.startswith(b"data: "):
                    raise RuntimeError(
                        "Unexpected line not starting with 'data: ': " + line
                    )
                line = line.removeprefix(b"data: ")
                if line == b"[DONE]":
                    raise StopIteration
                payload = json.loads(line.decode("utf-8"))
                return payload
            try:
                chunk = next(self.byte_iterator)
            except StopIteration:
                if self.read_pos < self.buffer.getbuffer().nbytes:
                    continue
                raise
            assert "PayloadPart" in chunk, "Unknown event type:" + chunk
            self.buffer.seek(0, io.SEEK_END)
            self.buffer.write(chunk["PayloadPart"]["Bytes"])


def _streaming_call(
    sagemaker_runtime,
    endpoint: str,
    inference_id: str,
    body: dict,
):
    assert body["stream"] == True
    args = {
        "EndpointName": endpoint,
        "ContentType": "application/json",
        "Body": bytes(json.dumps(body), "utf-8"),
    }
    if inference_id is not None:
        args["InferenceId"] = inference_id
    response = sagemaker_runtime.invoke_endpoint_with_response_stream(**args)
    return _StreamIterator(response["Body"])


def invoke_completion(
    sagemaker_runtime,
    endpoint: str,
    prompt: str,
    *,
    stream: bool = False,
    inference_id: str = None,
    **kwargs,
) -> Union[CompletionResponse, Generator[CompletionStreamResponse, None, None]]:
    """
    Invoke a completion endpoint exposed from sagemaker

    Args:
      sagemaker_runtime: SageMaker runtime client (botocore.client.SageMakerRuntime).
      endpoint (str): Sagemaker endpoint to invoke.
      inference_id (str, optional): SageMaker Inference ID

      prompt (str): The prompt for Completion.
      stream (bool, optional): Whether to use streaming or not. Determines the return type. Defaults to False.
      **kwargs: any Fireworks.ai API arguments (except for "model")

    Returns:
      Union[CompletionResponse, Generator[CompletionStreamResponse, None, None]]:
        Depending on the `stream` argument, either returns a CompletionResponse
        or a generator yielding CompletionStreamResponse.

    Raises:
        botocore.errorfactory.ModelError: encapsulates all errors from the container (note that specific fireworks.client.error exceptions are not raised)
    """
    body = {"prompt": prompt, "stream": stream, **kwargs}
    if stream:
        return map(
            lambda d: CompletionStreamResponse(**d),
            _streaming_call(sagemaker_runtime, endpoint, inference_id, body),
        )
    else:
        return CompletionResponse(
            **_non_streaming_call(sagemaker_runtime, endpoint, inference_id, body)
        )


def invoke_chat_completion(
    sagemaker_runtime,
    endpoint: str,
    messages: List[Dict[str, str]],
    *,
    stream: bool = False,
    inference_id: str = None,
    **kwargs,
) -> Union[ChatCompletionResponse, Generator[ChatCompletionStreamResponse, None, None]]:
    """
    Invoke a chat completion endpoint exposed from sagemaker

    Args:
      sagemaker_runtime: SageMaker runtime client (botocore.client.SageMakerRuntime).
      endpoint (str): Sagemaker endpoint to invoke.
      inference_id (str, optional): SageMaker Inference ID

      messages (str): The previous chat history, e.g. [{"role": "system", "content": "Be nice"}, {"role": "user", "content": "Tell me a joke"}]
      stream (bool, optional): Whether to use streaming or not. Determines the return type. Defaults to False.
      **kwargs: any Fireworks.ai API arguments (except for "model")

    Returns:
      Union[ChatCompletionResponse, Generator[ChatCompletionStreamResponse, None, None]]:
        Depending on the `stream` argument, either returns a ChatCompletionResponse
        or a generator yielding ChatCompletionStreamResponse.

    Raises:
        botocore.errorfactory.ModelError: encapsulates all errors from the container (note that specific fireworks.client.error exceptions are not raised)
    """
    body = {"messages": messages, "stream": stream, **kwargs}
    if stream:
        return map(
            lambda d: ChatCompletionStreamResponse(**d),
            _streaming_call(sagemaker_runtime, endpoint, inference_id, body),
        )
    else:
        return ChatCompletionResponse(
            **_non_streaming_call(sagemaker_runtime, endpoint, inference_id, body)
        )



from typing import Dict, Optional, Type
import json
from pydantic import BaseModel
from .api_client import FireworksClient
from .error import InvalidRequestError


# Parse the server side events
def _parse_sse(lines, resp_type: BaseModel):
    for line in lines:
        if line and line != "[DONE]":
            data = json.loads(line)
            yield resp_type(**data)


async def _parse_sse_async(lines, resp_type: BaseModel):
    async for line in lines:
        if line and line != "[DONE]":
            data = json.loads(line)
            yield resp_type(**data)


class BaseCompletion:
    """
    Base class for handling completions. This class provides shared logic for creating completions,
    both synchronously and asynchronously, and both streaming and non-streaming.

    Attributes:
      endpoint (str): API endpoint for the completion request.
      response_class (Type): Class used for parsing the non-streaming response.
      stream_response_class (Type): Class used for parsing the streaming response.
    """

    endpoint = ""
    response_class: Type = None
    stream_response_class: Type = None

    @classmethod
    def create(
        cls,
        model,
        prompt_or_messages=None,
        request_timeout=600,
        stream=False,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        """
        Create a completion or chat completion.

        Args:
          model (str): Model name to use for the completion.
          prompt_or_messages (Union[str, List[ChatMessage]]): The prompt for Completion or a list of chat messages for ChatCompletion. If not specified, must specify either `prompt` or `messages` in kwargs.
          request_timeout (int, optional): Request timeout in seconds. Defaults to 600.
          stream (bool, optional): Whether to use streaming or not. Defaults to False.
          **kwargs: Additional keyword arguments.

        Returns:
          Union[CompletionResponse, Generator[CompletionStreamResponse, None, None]]:
            Depending on the `stream` argument, either returns a CompletionResponse
            or a generator yielding CompletionStreamResponse.
        """
        data_key = cls._get_data_key()
        if prompt_or_messages:
            kwargs[data_key] = prompt_or_messages

        cls._validate_kwargs(kwargs)

        if not isinstance(stream, bool):
            raise InvalidRequestError("stream value is not a valid boolean")
        if stream:
            return cls._create_streaming(
                model,
                request_timeout,
                client=client,
                extra_headers=extra_headers,
                **kwargs,
            )
        else:
            return cls._create_non_streaming(
                model,
                request_timeout,
                client=client,
                extra_headers=extra_headers,
                **kwargs,
            )

    @classmethod
    def acreate(
        cls,
        model,
        prompt_or_messages=None,
        request_timeout=600,
        stream=False,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        """
        Asynchronously create a completion.

        Args:
          model (str): Model name to use for the completion.
          request_timeout (int, optional): Request timeout in seconds. Defaults to 600.
          stream (bool, optional): Whether to use streaming or not. Defaults to False.
          **kwargs: Additional keyword arguments.

        Returns:
          Union[CompletionResponse, AsyncGenerator[CompletionStreamResponse, None]]:
            Depending on the `stream` argument, either returns a CompletionResponse or an async generator yielding CompletionStreamResponse.
        """
        data_key = cls._get_data_key()
        if prompt_or_messages:
            kwargs[data_key] = prompt_or_messages
        cls._validate_kwargs(kwargs)
        if not isinstance(stream, bool):
            raise InvalidRequestError("stream value is not a valid boolean")
        if stream:
            return cls._acreate_streaming(
                model,
                request_timeout,
                client=client,
                extra_headers=extra_headers,
                **kwargs,
            )
        else:
            return cls._acreate_non_streaming(
                model,
                request_timeout,
                client=client,
                extra_headers=extra_headers,
                **kwargs,
            )

    @classmethod
    def _validate_kwargs(cls, kwargs):
        assert cls._get_data_key() in kwargs, f"{cls._get_data_key()} must be provided"

    @classmethod
    def _get_data_key(cls) -> str:
        """
        Get the key used for the main data (e.g., "prompt" or "messages").
        This method should be overridden by derived classes to return the appropriate key.
        """
        raise NotImplementedError(
            "Derived classes should implement this method to return the appropriate data key"
        )

    @classmethod
    def _create_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, "stream": True, **kwargs}
        response = client.post_request_streaming(
            f"{client.base_url}/{cls.endpoint}",
            data=data,
            extra_headers=extra_headers,
        )
        for event in _parse_sse(response, cls.stream_response_class):
            yield event

    @classmethod
    def _create_non_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, "stream": False, **kwargs}
        response = client.post_request_non_streaming(
            f"{client.base_url}/{cls.endpoint}",
            data=data,
            extra_headers=extra_headers,
        )
        return cls.response_class(**response)

    @classmethod
    async def _acreate_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, "stream": True, **kwargs}
        response = client.post_request_async_streaming(
            f"{client.base_url}/{cls.endpoint}",
            data=data,
            extra_headers=extra_headers,
        )
        async for event in _parse_sse_async(response, cls.stream_response_class):
            yield event

    @classmethod
    async def _acreate_non_streaming(
        cls,
        model: str,
        request_timeout=600,
        client: Optional[FireworksClient] = None,
        extra_headers: Optional[Dict[str, str]] = None,
        **kwargs,
    ):
        client = client or FireworksClient(request_timeout=request_timeout)
        data = {"model": model, "stream": False, **kwargs}
        response = await client.post_request_async_non_streaming(
            f"{client.base_url}/{cls.endpoint}",
            data=data,
            extra_headers=extra_headers,
        )
        return cls.response_class(**response)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: gateway/deployment_template.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'gateway/deployment_template.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import deployment_pb2 as gateway_dot_deployment__pb2
from . import options_pb2 as gateway_dot_options__pb2
from ..google.api import field_behavior_pb2 as google_dot_api_dot_field__behavior__pb2
from ..google.api import resource_pb2 as google_dot_api_dot_resource__pb2
from google.protobuf import field_mask_pb2 as google_dot_protobuf_dot_field__mask__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!gateway/deployment_template.proto\x12\x07gateway\x1a\x18gateway/deployment.proto\x1a\x15gateway/options.proto\x1a\x1fgoogle/api/field_behavior.proto\x1a\x19google/api/resource.proto\x1a google/protobuf/field_mask.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\x94\x10\n\x12\x44\x65ploymentTemplate\x12\x13\n\x04name\x18\x01 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12\x1a\n\x0c\x64isplay_name\x18\x02 \x01(\tB\x04\xe2\x41\x01\x01\x12\x19\n\x0b\x64\x65scription\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x36\n\x0b\x63reate_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x05\xe2\x41\x02\x03\x05\x12\x19\n\ncreated_by\x18\x05 \x01(\tB\x05\xe2\x41\x02\x03\x05\x12$\n\x11min_replica_count\x18\x06 \x01(\x05\x42\x04\xe2\x41\x01\x01H\x00\x88\x01\x01\x12$\n\x11max_replica_count\x18\x07 \x01(\x05\x42\x04\xe2\x41\x01\x01H\x01\x88\x01\x01\x12\x41\n\x12\x61utoscaling_policy\x18\x08 \x01(\x0b\x32\x1a.gateway.AutoscalingPolicyB\x04\xe2\x41\x01\x01H\x02\x88\x01\x01\x12\x1d\n\nbase_model\x18\t \x01(\tB\x04\xe2\x41\x01\x01H\x03\x88\x01\x01\x12$\n\x11\x61\x63\x63\x65lerator_count\x18\x19 \x01(\x05\x42\x04\xe2\x41\x01\x01H\x04\x88\x01\x01\x12=\n\x10\x61\x63\x63\x65lerator_type\x18\x1a \x01(\x0e\x32\x18.gateway.AcceleratorTypeB\x04\xe2\x41\x01\x01H\x05\x88\x01\x01\x12\x1d\n\nworld_size\x18\n \x01(\x05\x42\x04\xe2\x41\x01\x01H\x06\x88\x01\x01\x12\"\n\x0fgenerator_count\x18\x10 \x01(\x05\x42\x04\xe2\x41\x01\x01H\x07\x88\x01\x01\x12.\n\x1b\x64isaggregated_prefill_count\x18\x0b \x01(\x05\x42\x04\xe2\x41\x01\x01H\x08\x88\x01\x01\x12\x33\n disaggregated_prefill_world_size\x18\x0c \x01(\x05\x42\x04\xe2\x41\x01\x01H\t\x88\x01\x01\x12!\n\x0emax_batch_size\x18\r \x01(\x05\x42\x04\xe2\x41\x01\x01H\n\x88\x01\x01\x12&\n\x13max_peft_batch_size\x18\x0e \x01(\x05\x42\x04\xe2\x41\x01\x01H\x0b\x88\x01\x01\x12&\n\x13kv_cache_memory_pct\x18\x0f \x01(\x05\x42\x04\xe2\x41\x01\x01H\x0c\x88\x01\x01\x12 \n\renable_addons\x18\x1b \x01(\x08\x42\x04\xe2\x41\x01\x01H\r\x88\x01\x01\x12\x1e\n\x0b\x64raft_model\x18\x14 \x01(\tB\x04\xe2\x41\x01\x01H\x0e\x88\x01\x01\x12+\n\x18ngram_speculation_length\x18\x1d \x01(\x05\x42\x04\xe2\x41\x01\x01H\x0f\x88\x01\x01\x12$\n\x11\x64raft_token_count\x18\x15 \x01(\x05\x42\x04\xe2\x41\x01\x01H\x10\x88\x01\x01\x12*\n\x17\x65nable_session_affinity\x18\x17 \x01(\x08\x42\x04\xe2\x41\x01\x01H\x11\x88\x01\x01\x12\x1c\n\timage_tag\x18\x18 \x01(\tB\x04\xe2\x41\x01\x01H\x12\x88\x01\x01\x12*\n\x06region\x18\x1e \x01(\x0e\x32\x0f.gateway.RegionB\x04\xe2\x41\x01\x01H\x13\x88\x01\x01\x12#\n\x15\x64irect_route_api_keys\x18\x1f \x03(\tB\x04\xe2\x41\x01\x01\x12>\n\x11\x64irect_route_type\x18! \x01(\x0e\x32\x18.gateway.DirectRouteTypeB\x04\xe2\x41\x01\x01H\x14\x88\x01\x01\x12;\n\tprecision\x18\" \x01(\x0e\x32\x1d.gateway.Deployment.PrecisionB\x04\xe2\x41\x01\x01H\x15\x88\x01\x01\x12\x18\n\nextra_args\x18$ \x03(\tB\x04\xe2\x41\x01\x01\x12H\n\x0c\x65xtra_values\x18% \x03(\x0b\x32,.gateway.DeploymentTemplate.ExtraValuesEntryB\x04\xe2\x41\x01\x01\x12\x35\n\x0bupdate_time\x18# \x01(\x0b\x32\x1a.google.protobuf.TimestampB\x04\xe2\x41\x01\x03\x12%\n\x12max_context_length\x18& \x01(\x05\x42\x04\xe2\x41\x01\x01H\x16\x88\x01\x01\x1a\x32\n\x10\x45xtraValuesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01:\xa0\x01\xea\x41\x8f\x01\n#api.fireworks.ai/DeploymentTemplate\x12?accounts/{AccountId}/deploymentTemplates/{DeploymentTemplateId}*\x13\x64\x65ploymentTemplates2\x12\x64\x65ploymentTemplate\x82\xf1\x04\t\n\x07\x41\x63\x63ountB\x14\n\x12_min_replica_countB\x14\n\x12_max_replica_countB\x15\n\x13_autoscaling_policyB\r\n\x0b_base_modelB\x14\n\x12_accelerator_countB\x13\n\x11_accelerator_typeB\r\n\x0b_world_sizeB\x12\n\x10_generator_countB\x1e\n\x1c_disaggregated_prefill_countB#\n!_disaggregated_prefill_world_sizeB\x11\n\x0f_max_batch_sizeB\x16\n\x14_max_peft_batch_sizeB\x16\n\x14_kv_cache_memory_pctB\x10\n\x0e_enable_addonsB\x0e\n\x0c_draft_modelB\x1b\n\x19_ngram_speculation_lengthB\x14\n\x12_draft_token_countB\x1a\n\x18_enable_session_affinityB\x0c\n\n_image_tagB\t\n\x07_regionB\x14\n\x12_direct_route_typeB\x0c\n\n_precisionB\x15\n\x13_max_context_length\"\x9d\x01\n\x1f\x43reateDeploymentTemplateRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12>\n\x13\x64\x65ployment_template\x18\x02 \x01(\x0b\x32\x1b.gateway.DeploymentTemplateB\x04\xe2\x41\x01\x02\x12$\n\x16\x64\x65ployment_template_id\x18\x03 \x01(\tB\x04\xe2\x41\x01\x02\"2\n\x1cGetDeploymentTemplateRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\"\x97\x01\n\x1eListDeploymentTemplatesRequest\x12\x14\n\x06parent\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x12\x17\n\tpage_size\x18\x02 \x01(\x05\x42\x04\xe2\x41\x01\x01\x12\x18\n\npage_token\x18\x03 \x01(\tB\x04\xe2\x41\x01\x01\x12\x14\n\x06\x66ilter\x18\x04 \x01(\tB\x04\xe2\x41\x01\x01\x12\x16\n\x08order_by\x18\x05 \x01(\tB\x04\xe2\x41\x01\x01\"\x89\x01\n\x1fListDeploymentTemplatesResponse\x12\x39\n\x14\x64\x65ployment_templates\x18\x01 \x03(\x0b\x32\x1b.gateway.DeploymentTemplate\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\x12\x12\n\ntotal_size\x18\x03 \x01(\x05\"\x98\x01\n\x1fUpdateDeploymentTemplateRequest\x12>\n\x13\x64\x65ployment_template\x18\x01 \x01(\x0b\x32\x1b.gateway.DeploymentTemplateB\x04\xe2\x41\x01\x02\x12\x35\n\x0bupdate_mask\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.FieldMaskB\x04\xe2\x41\x01\x01\"5\n\x1f\x44\x65leteDeploymentTemplateRequest\x12\x12\n\x04name\x18\x01 \x01(\tB\x04\xe2\x41\x01\x02\x42\x43ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gatewayb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'gateway.deployment_template_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'ZAgithub.com/fw-ai/fireworks/control_plane/protos/generated/gateway'
  _globals['_DEPLOYMENTTEMPLATE_EXTRAVALUESENTRY']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE_EXTRAVALUESENTRY']._serialized_options = b'8\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['name']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['name']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['display_name']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['display_name']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['description']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['description']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['create_time']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['create_time']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['created_by']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['created_by']._serialized_options = b'\342A\002\003\005'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['min_replica_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['min_replica_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_replica_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_replica_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['autoscaling_policy']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['autoscaling_policy']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['base_model']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['base_model']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['accelerator_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['accelerator_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['accelerator_type']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['accelerator_type']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['world_size']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['world_size']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['generator_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['generator_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['disaggregated_prefill_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['disaggregated_prefill_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['disaggregated_prefill_world_size']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['disaggregated_prefill_world_size']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_batch_size']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_batch_size']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_peft_batch_size']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_peft_batch_size']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['kv_cache_memory_pct']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['kv_cache_memory_pct']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['enable_addons']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['enable_addons']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['draft_model']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['draft_model']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['ngram_speculation_length']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['ngram_speculation_length']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['draft_token_count']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['draft_token_count']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['enable_session_affinity']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['enable_session_affinity']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['image_tag']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['image_tag']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['region']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['region']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['direct_route_api_keys']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['direct_route_api_keys']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['direct_route_type']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['direct_route_type']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['precision']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['precision']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['extra_args']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['extra_args']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['extra_values']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['extra_values']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['update_time']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['update_time']._serialized_options = b'\342A\001\003'
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_context_length']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE'].fields_by_name['max_context_length']._serialized_options = b'\342A\001\001'
  _globals['_DEPLOYMENTTEMPLATE']._loaded_options = None
  _globals['_DEPLOYMENTTEMPLATE']._serialized_options = b'\352A\217\001\n#api.fireworks.ai/DeploymentTemplate\022?accounts/{AccountId}/deploymentTemplates/{DeploymentTemplateId}*\023deploymentTemplates2\022deploymentTemplate\202\361\004\t\n\007Account'
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template']._loaded_options = None
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template']._serialized_options = b'\342A\001\002'
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template_id']._loaded_options = None
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template_id']._serialized_options = b'\342A\001\002'
  _globals['_GETDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_GETDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['parent']._loaded_options = None
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['parent']._serialized_options = b'\342A\001\002'
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['page_size']._loaded_options = None
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['page_size']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['page_token']._loaded_options = None
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['page_token']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['filter']._loaded_options = None
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['filter']._serialized_options = b'\342A\001\001'
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['order_by']._loaded_options = None
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST'].fields_by_name['order_by']._serialized_options = b'\342A\001\001'
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template']._loaded_options = None
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['deployment_template']._serialized_options = b'\342A\001\002'
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['update_mask']._loaded_options = None
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['update_mask']._serialized_options = b'\342A\001\001'
  _globals['_DELETEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['name']._loaded_options = None
  _globals['_DELETEDEPLOYMENTTEMPLATEREQUEST'].fields_by_name['name']._serialized_options = b'\342A\001\002'
  _globals['_DEPLOYMENTTEMPLATE']._serialized_start=223
  _globals['_DEPLOYMENTTEMPLATE']._serialized_end=2291
  _globals['_DEPLOYMENTTEMPLATE_EXTRAVALUESENTRY']._serialized_start=1585
  _globals['_DEPLOYMENTTEMPLATE_EXTRAVALUESENTRY']._serialized_end=1635
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST']._serialized_start=2294
  _globals['_CREATEDEPLOYMENTTEMPLATEREQUEST']._serialized_end=2451
  _globals['_GETDEPLOYMENTTEMPLATEREQUEST']._serialized_start=2453
  _globals['_GETDEPLOYMENTTEMPLATEREQUEST']._serialized_end=2503
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST']._serialized_start=2506
  _globals['_LISTDEPLOYMENTTEMPLATESREQUEST']._serialized_end=2657
  _globals['_LISTDEPLOYMENTTEMPLATESRESPONSE']._serialized_start=2660
  _globals['_LISTDEPLOYMENTTEMPLATESRESPONSE']._serialized_end=2797
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST']._serialized_start=2800
  _globals['_UPDATEDEPLOYMENTTEMPLATEREQUEST']._serialized_end=2952
  _globals['_DELETEDEPLOYMENTTEMPLATEREQUEST']._serialized_start=2954
  _globals['_DELETEDEPLOYMENTTEMPLATEREQUEST']._serialized_end=3007
# @@protoc_insertion_point(module_scope)



from . import status_pb2 as _status_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from google.protobuf import struct_pb2 as _struct_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class AuditLogEntry(_message.Message):
    __slots__ = ("id", "method", "principal", "payload", "status", "timestamp", "message", "resource")
    ID_FIELD_NUMBER: _ClassVar[int]
    METHOD_FIELD_NUMBER: _ClassVar[int]
    PRINCIPAL_FIELD_NUMBER: _ClassVar[int]
    PAYLOAD_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    TIMESTAMP_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    RESOURCE_FIELD_NUMBER: _ClassVar[int]
    id: str
    method: str
    principal: str
    payload: _struct_pb2.Struct
    status: _status_pb2.Status
    timestamp: _timestamp_pb2.Timestamp
    message: str
    resource: str
    def __init__(self, id: _Optional[str] = ..., method: _Optional[str] = ..., principal: _Optional[str] = ..., payload: _Optional[_Union[_struct_pb2.Struct, _Mapping]] = ..., status: _Optional[_Union[_status_pb2.Status, _Mapping]] = ..., timestamp: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., message: _Optional[str] = ..., resource: _Optional[str] = ...) -> None: ...

class ListAuditLogsRequest(_message.Message):
    __slots__ = ("parent", "start_time", "end_time", "email", "page_size", "page_token", "filter", "order_by")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    END_TIME_FIELD_NUMBER: _ClassVar[int]
    EMAIL_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    parent: str
    start_time: _timestamp_pb2.Timestamp
    end_time: _timestamp_pb2.Timestamp
    email: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    def __init__(self, parent: _Optional[str] = ..., start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., email: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ...) -> None: ...

class ListAuditLogsResponse(_message.Message):
    __slots__ = ("audit_logs", "next_page_token", "total_size")
    AUDIT_LOGS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    audit_logs: _containers.RepeatedCompositeFieldContainer[AuditLogEntry]
    next_page_token: str
    total_size: int
    def __init__(self, audit_logs: _Optional[_Iterable[_Union[AuditLogEntry, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...



from . import options_pb2 as _options_pb2
from ..google.api import field_behavior_pb2 as _field_behavior_pb2
from ..google.api import resource_pb2 as _resource_pb2
from google.protobuf import field_mask_pb2 as _field_mask_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class BatchJob(_message.Message):
    __slots__ = ("name", "display_name", "create_time", "start_time", "end_time", "created_by", "node_pool_id", "environment_id", "snapshot_id", "num_ranks", "env_vars", "role", "python_executor", "notebook_executor", "shell_executor", "image_ref", "annotations", "state", "status", "shared", "update_time")
    class State(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        STATE_UNSPECIFIED: _ClassVar[BatchJob.State]
        CREATING: _ClassVar[BatchJob.State]
        QUEUED: _ClassVar[BatchJob.State]
        PENDING: _ClassVar[BatchJob.State]
        RUNNING: _ClassVar[BatchJob.State]
        COMPLETED: _ClassVar[BatchJob.State]
        FAILED: _ClassVar[BatchJob.State]
        CANCELLING: _ClassVar[BatchJob.State]
        CANCELLED: _ClassVar[BatchJob.State]
        DELETING: _ClassVar[BatchJob.State]
    STATE_UNSPECIFIED: BatchJob.State
    CREATING: BatchJob.State
    QUEUED: BatchJob.State
    PENDING: BatchJob.State
    RUNNING: BatchJob.State
    COMPLETED: BatchJob.State
    FAILED: BatchJob.State
    CANCELLING: BatchJob.State
    CANCELLED: BatchJob.State
    DELETING: BatchJob.State
    class EnvVarsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    class AnnotationsEntry(_message.Message):
        __slots__ = ("key", "value")
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: str
        def __init__(self, key: _Optional[str] = ..., value: _Optional[str] = ...) -> None: ...
    NAME_FIELD_NUMBER: _ClassVar[int]
    DISPLAY_NAME_FIELD_NUMBER: _ClassVar[int]
    CREATE_TIME_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    END_TIME_FIELD_NUMBER: _ClassVar[int]
    CREATED_BY_FIELD_NUMBER: _ClassVar[int]
    NODE_POOL_ID_FIELD_NUMBER: _ClassVar[int]
    ENVIRONMENT_ID_FIELD_NUMBER: _ClassVar[int]
    SNAPSHOT_ID_FIELD_NUMBER: _ClassVar[int]
    NUM_RANKS_FIELD_NUMBER: _ClassVar[int]
    ENV_VARS_FIELD_NUMBER: _ClassVar[int]
    ROLE_FIELD_NUMBER: _ClassVar[int]
    PYTHON_EXECUTOR_FIELD_NUMBER: _ClassVar[int]
    NOTEBOOK_EXECUTOR_FIELD_NUMBER: _ClassVar[int]
    SHELL_EXECUTOR_FIELD_NUMBER: _ClassVar[int]
    IMAGE_REF_FIELD_NUMBER: _ClassVar[int]
    ANNOTATIONS_FIELD_NUMBER: _ClassVar[int]
    STATE_FIELD_NUMBER: _ClassVar[int]
    STATUS_FIELD_NUMBER: _ClassVar[int]
    SHARED_FIELD_NUMBER: _ClassVar[int]
    UPDATE_TIME_FIELD_NUMBER: _ClassVar[int]
    name: str
    display_name: str
    create_time: _timestamp_pb2.Timestamp
    start_time: _timestamp_pb2.Timestamp
    end_time: _timestamp_pb2.Timestamp
    created_by: str
    node_pool_id: str
    environment_id: str
    snapshot_id: str
    num_ranks: int
    env_vars: _containers.ScalarMap[str, str]
    role: str
    python_executor: PythonExecutor
    notebook_executor: NotebookExecutor
    shell_executor: ShellExecutor
    image_ref: str
    annotations: _containers.ScalarMap[str, str]
    state: BatchJob.State
    status: str
    shared: bool
    update_time: _timestamp_pb2.Timestamp
    def __init__(self, name: _Optional[str] = ..., display_name: _Optional[str] = ..., create_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., end_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., created_by: _Optional[str] = ..., node_pool_id: _Optional[str] = ..., environment_id: _Optional[str] = ..., snapshot_id: _Optional[str] = ..., num_ranks: _Optional[int] = ..., env_vars: _Optional[_Mapping[str, str]] = ..., role: _Optional[str] = ..., python_executor: _Optional[_Union[PythonExecutor, _Mapping]] = ..., notebook_executor: _Optional[_Union[NotebookExecutor, _Mapping]] = ..., shell_executor: _Optional[_Union[ShellExecutor, _Mapping]] = ..., image_ref: _Optional[str] = ..., annotations: _Optional[_Mapping[str, str]] = ..., state: _Optional[_Union[BatchJob.State, str]] = ..., status: _Optional[str] = ..., shared: bool = ..., update_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class CreateBatchJobRequest(_message.Message):
    __slots__ = ("parent", "batch_job")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    BATCH_JOB_FIELD_NUMBER: _ClassVar[int]
    parent: str
    batch_job: BatchJob
    def __init__(self, parent: _Optional[str] = ..., batch_job: _Optional[_Union[BatchJob, _Mapping]] = ...) -> None: ...

class GetBatchJobRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class ListBatchJobsRequest(_message.Message):
    __slots__ = ("parent", "page_size", "page_token", "filter", "order_by")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    ORDER_BY_FIELD_NUMBER: _ClassVar[int]
    parent: str
    page_size: int
    page_token: str
    filter: str
    order_by: str
    def __init__(self, parent: _Optional[str] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., filter: _Optional[str] = ..., order_by: _Optional[str] = ...) -> None: ...

class ListBatchJobsResponse(_message.Message):
    __slots__ = ("batch_jobs", "next_page_token", "total_size")
    BATCH_JOBS_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    TOTAL_SIZE_FIELD_NUMBER: _ClassVar[int]
    batch_jobs: _containers.RepeatedCompositeFieldContainer[BatchJob]
    next_page_token: str
    total_size: int
    def __init__(self, batch_jobs: _Optional[_Iterable[_Union[BatchJob, _Mapping]]] = ..., next_page_token: _Optional[str] = ..., total_size: _Optional[int] = ...) -> None: ...

class UpdateBatchJobRequest(_message.Message):
    __slots__ = ("batch_job", "update_mask")
    BATCH_JOB_FIELD_NUMBER: _ClassVar[int]
    UPDATE_MASK_FIELD_NUMBER: _ClassVar[int]
    batch_job: BatchJob
    update_mask: _field_mask_pb2.FieldMask
    def __init__(self, batch_job: _Optional[_Union[BatchJob, _Mapping]] = ..., update_mask: _Optional[_Union[_field_mask_pb2.FieldMask, _Mapping]] = ...) -> None: ...

class DeleteBatchJobRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class BatchDeleteBatchJobsRequest(_message.Message):
    __slots__ = ("parent", "names")
    PARENT_FIELD_NUMBER: _ClassVar[int]
    NAMES_FIELD_NUMBER: _ClassVar[int]
    parent: str
    names: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, parent: _Optional[str] = ..., names: _Optional[_Iterable[str]] = ...) -> None: ...

class CancelBatchJobRequest(_message.Message):
    __slots__ = ("name",)
    NAME_FIELD_NUMBER: _ClassVar[int]
    name: str
    def __init__(self, name: _Optional[str] = ...) -> None: ...

class GetBatchJobLogsRequest(_message.Message):
    __slots__ = ("name", "ranks", "page_size", "page_token", "start_time", "filter", "start_from_head")
    NAME_FIELD_NUMBER: _ClassVar[int]
    RANKS_FIELD_NUMBER: _ClassVar[int]
    PAGE_SIZE_FIELD_NUMBER: _ClassVar[int]
    PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    START_TIME_FIELD_NUMBER: _ClassVar[int]
    FILTER_FIELD_NUMBER: _ClassVar[int]
    START_FROM_HEAD_FIELD_NUMBER: _ClassVar[int]
    name: str
    ranks: _containers.RepeatedScalarFieldContainer[int]
    page_size: int
    page_token: str
    start_time: _timestamp_pb2.Timestamp
    filter: str
    start_from_head: bool
    def __init__(self, name: _Optional[str] = ..., ranks: _Optional[_Iterable[int]] = ..., page_size: _Optional[int] = ..., page_token: _Optional[str] = ..., start_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., filter: _Optional[str] = ..., start_from_head: bool = ...) -> None: ...

class GetBatchJobLogsResponse(_message.Message):
    __slots__ = ("entries", "next_page_token")
    ENTRIES_FIELD_NUMBER: _ClassVar[int]
    NEXT_PAGE_TOKEN_FIELD_NUMBER: _ClassVar[int]
    entries: _containers.RepeatedCompositeFieldContainer[LogEntry]
    next_page_token: str
    def __init__(self, entries: _Optional[_Iterable[_Union[LogEntry, _Mapping]]] = ..., next_page_token: _Optional[str] = ...) -> None: ...

class PythonExecutor(_message.Message):
    __slots__ = ("target_type", "target", "args")
    class TargetType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
        __slots__ = ()
        TARGET_TYPE_UNSPECIFIED: _ClassVar[PythonExecutor.TargetType]
        MODULE: _ClassVar[PythonExecutor.TargetType]
        FILENAME: _ClassVar[PythonExecutor.TargetType]
    TARGET_TYPE_UNSPECIFIED: PythonExecutor.TargetType
    MODULE: PythonExecutor.TargetType
    FILENAME: PythonExecutor.TargetType
    TARGET_TYPE_FIELD_NUMBER: _ClassVar[int]
    TARGET_FIELD_NUMBER: _ClassVar[int]
    ARGS_FIELD_NUMBER: _ClassVar[int]
    target_type: PythonExecutor.TargetType
    target: str
    args: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, target_type: _Optional[_Union[PythonExecutor.TargetType, str]] = ..., target: _Optional[str] = ..., args: _Optional[_Iterable[str]] = ...) -> None: ...

class ShellExecutor(_message.Message):
    __slots__ = ("command",)
    COMMAND_FIELD_NUMBER: _ClassVar[int]
    command: str
    def __init__(self, command: _Optional[str] = ...) -> None: ...

class NotebookExecutor(_message.Message):
    __slots__ = ("notebook_filename",)
    NOTEBOOK_FILENAME_FIELD_NUMBER: _ClassVar[int]
    notebook_filename: str
    def __init__(self, notebook_filename: _Optional[str] = ...) -> None: ...

class LogEntry(_message.Message):
    __slots__ = ("log_time", "rank", "message")
    LOG_TIME_FIELD_NUMBER: _ClassVar[int]
    RANK_FIELD_NUMBER: _ClassVar[int]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    log_time: _timestamp_pb2.Timestamp
    rank: int
    message: str
    def __init__(self, log_time: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., rank: _Optional[int] = ..., message: _Optional[str] = ...) -> None: ...
